function [img,H] = CTrecon(prj,varargin)
%  Reconstruct an image from parallel projection data using the convolution-backprojection algorithm
%  function [img,H] = CTrecon(prj,varargin)
%
%  for a discussion of this function including examples see: http://aprendtech.com/wordpress/?p=74
%
%  Has the following improvements over iradon:
%
%  1. Fixes a bug in iradon that results in offsets from
%   the true values in the reconstructed image. Does this by computing 
%   the filter transfer function as transform of direct space function
%   instead of transforming a ramp in frequency.
%  See Ch. 3 of "Principles of Computerized Tomographic Imaging" by 
%  A. C. Kak and Malcolm Slaney, IEEE Press, 1988. 
%  available online at http://www.slaney.org/pct/pct-toc.html
% 
%  2. Uses mex functions based on  iradon_speedy to back project so it is faster
%
%  3. handles complex projections to return a complex reconstructed image - see zBackproject.c
%
%  4. regularizes the interface to ('parameter_name', parameter_value) syntax
%  
%  inputs:
%      prj: (required) the projection data matrix of size  (number of lines)X(number_of_angles)
%      angles: optional = angles are equally spaced from 0 to 180 degrees but do not wrap around
%                    number of angles is size(prj,2)
%
%      interpolation: optional = 'linear', a string specifying the interpolation in the backprojection
%            'linear'  - linear interpolation (default)
%            'nearest neighbor' - nearest neighbor interpolation 
%    filter_type:(optional = 'Ram-Lak') a string that specifies the filter used. Possibilities are          
%         'Ram-Lak'     The cropped Ramachandran-Lakshimrayanan or ramp filter_type (default).  
%         'Shepp-Logan' The Shepp-Logan filter_type: a ramp times a sinc
%         'Cosine'      a ramp times cosine from 0 to the Nyquist frequency.
%         'Hamming'     a ramp times the Hamming filter_type.
%         'Hann'        a ramp times the Hann filter_type.
%   
%   freq_cutoff: (optional = 1) sets the filter to 0 starting at freq_cutoff*freq_nyquist 
%
%  outputs:
%      img: the reconstructed image
%      H: the transfer function of the filter
% 
%   The image output size is determined from the length of the projections:
%
%       OUTPUT_SIZE = 2*floor(size(prj,1)/(2*sqrt(2)))
%
%   Example: see http://aprendtech.com/wordpress/?p=74 for other data
%   -------
%    P = phantom(128);
%    R = radon(P,0:179);
%    img = CTrecon(R,'filter_type','cosine','freq_cutoff',0.9);
%    figure, imshow(img,[-.05 .3])
%  
%  Copyright (c) 2011, Aprend Technology and Robert E. Alvarez
%  All rights reserved.
%  
%  Redistribution and use in source and binary forms, with or without 
%  modification, are permitted provided that the following conditions are 
%  met:
%  
%      * Redistributions of source code must retain the above copyright 
%        notice, this list of conditions and the following disclaimer.
%      * Redistributions in binary form must reproduce the above copyright 
%        notice, this list of conditions and the following disclaimer in 
%        the documentation and/or other materials provided with the distribution
%        
%  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
%  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
%  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
%  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
%  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
%  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
%  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
%  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
%  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
%  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
%  POSSIBILITY OF SUCH DAMAGE.

   % handle the optional arguments
nreqargs = 1;
if nargin<nreqargs
   fprintf('syntax: [img,H] = CTrecon(prj,varargin)'); 
   error('%d is too few arguments',nargin);
end

[nlines, nangles] =size(prj);   
angles = linspace(0,180,nangles+1);
angles = angles(1:(end-1) ); % make sure angles do not wrap around 
interpolation = 'linear';
filter_type = 'Ram-Lak';
freq_cutoff = 1;
  
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'angles';   angles=varargin{i+1};  
        if numel(angles) ~= nangles
           error('CTrecon: length of angles array %d must equal size(prj,2) %d',numel(angles), nangles);
        end   
         i=i+1;
     case 'interpolation';         interpolation=varargin{i+1};
         if ~ischar(interpolation) 
           error('CTrecon: interpolation parameter must be a string');
        end   
         i=i+1;
     case 'filter_type';         filter_type=varargin{i+1};
         if ~ischar(interpolation) 
           error('CTrecon: filter_type must be a string');
        end   
         i=i+1;
     case 'freq_cutoff';         freq_cutoff=varargin{i+1};
         if (freq_cutoff<0)  || (freq_cutoff>1)
           error('CTrecon: freq_cutoff %g must be between 0 and 1', freq_cutoff);
        end   
         i=i+1;
    otherwise
      error('CTrecon: Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end


    % prep the FFT of the filter function
H = MakeFilter(filter_type, nlines, freq_cutoff);
prj(numel(H),1)=0;  % Zero pad projections 


prj_fft = fft(prj);    % prj_fft holds fft of projections

    % multiply data for each angle times H for the convolution
for i = 1:size(prj_fft,2)
   prj_fft(:,i) = prj_fft(:,i).*H; % frequency domain filtering
end

prj_filt = ifft(prj_fft);     %  prj_filt is the filtered projections. 
            %  note do not use real()  since may have complex projections
prj_filt( (nlines+1):end,:) = [];   % truncate the filtered projections

img_side = 2*floor( nlines/(2*sqrt(2)) );  
img = zeros(img_side,class(prj_filt));        % Allocate memory for the image.

   % prep data for backprojection
      % convert angles to radians
angles = angles*(pi/180);         
costheta = cos(angles);
sintheta = sin(angles);

    % Zero pad the projections to size 1+2*ceil(img_side/sqrt(2)) if this
    % quantity is greater than the length of the projections
img_diag = 2*ceil(img_side/sqrt(2))+1;  % largest distance through image.
if size(prj_filt,1) < img_diag 
   rz = img_diag - size(prj_filt,1);  % how many rows of zeros
   prj_filt = [zeros(ceil(rz/2),size(prj_filt,2)); prj_filt; zeros(floor(rz/2),size(prj_filt,2))];
end

    % Backprojection - use mex functions
if isreal(prj)
  is_complex = false;
else
  is_complex = true;
end

switch interpolation
    case 'nearest neighbor'
        if is_complex
            img = zBackproject(prj_filt, costheta, sintheta, img_side, 0);
        else
            img = Backproject(prj_filt, costheta, sintheta, img_side, 0);
        end

    case 'linear'
        if is_complex
            img = zBackproject(prj_filt, costheta, sintheta, img_side, 1);
        else
            img = Backproject(prj_filt, costheta, sintheta, img_side, 1);
        end

     otherwise
       error('interpolation method %s not supported',interp);


end

img = img*pi/numel(angles);


% ------------- Sub Functions----------------

    function h_fft = MakeFilter(filter_type, nlines, freq_cutoff)
        % Returns the Fourier Transform of the filter
        % used to convolve the projections
        %
        % inputs:   
        %   filter_type the string specifying the filter 
        %   nlines    - the number of lines of the projections
        %   freq_cutoff      - the fraction of frequencies below the nyquist to pass
        %
        %  outputs:
        %    h_fft   - the transformed filter to use on the projections
        
        
        
             % create filter by transforming the direct space formula (Eq. 61 of Kak and Slaney)
                % the filter order is the next highest power of 2.
        order = max(64,2^nextpow2(2*nlines));
        ns = 0:(order/2);
        h = zeros(size(ns));
        h(1) = 0.25;
        h(ns(2:2:end)+1) = (-1/pi^2)./ns(2:2:end).^2;
        h = [h ,  h(end-1:-1:2)]; % make into form for circular convolve
        filt_all = fft(h);
           
           % take apart to apply apodizing filter and adjust norm to be equal to norm of bowtie filter
        w = (2*pi/order)*ns;   % normalized frequency axis up to Nyquist 
        h_fft = real(filt_all(1:numel(w)));
        
           % apply apodizing filter
        switch lower(filter_type)
            case 'ram-lak'
               % Do nothing
            case 'shepp-logan'
               % be careful not to divide by 0:
               h_fft(2:end) = h_fft(2:end) .* (sin(w(2:end)/(2*freq_cutoff))./(w(2:end)/(2*freq_cutoff)));
            case 'cosine'
               h_fft(2:end) = h_fft(2:end) .* cos(w(2:end)/(2*freq_cutoff));
            case 'hamming'  
               h_fft(2:end) = h_fft(2:end) .* (.54 + .46 * cos(w(2:end)/freq_cutoff));
            case 'hann'
               h_fft(2:end) = h_fft(2:end) .*(1+cos(w(2:end)./freq_cutoff)) / 2;
                    
            otherwise
               error('CTrecon: %s is unknown filter type',filter_type);
        end
        
        h_fft(w>pi*freq_cutoff) = 0;                      % Crop the frequency response
        h_fft = [h_fft' ; h_fft(end-1:-1:2)'];    % extend filter to negative frequencies per FFT order